package net.rumati.logging.muffero;

import java.util.Date;

public class LogEvent
{
    private final String loggerName;
    private final Priority priority;
    private final String message;
    private final Throwable throwable;
    private final long time;
    private final Thread thread;
    private final StackTraceElement[] stackTrace;

    private LogEvent(String loggerName, Priority priority, String message, Throwable throwable, long time, Thread thread,
                    StackTraceElement[] stackTrace)
    {
        this.loggerName = loggerName;
        this.priority = priority;
        this.message = message;
        this.throwable = throwable;
        this.time = time;
        this.thread = thread;
        this.stackTrace = stackTrace;
    }

    LogEvent(String fqcn, String loggerName, Priority priority, String message, Throwable throwable)
    {
        this.loggerName = loggerName;
        this.priority = priority;
        this.message = message;
        this.throwable = throwable;
        this.time = new Date().getTime();
        this.thread = Thread.currentThread();

        StackTraceElement[] st = Thread.currentThread().getStackTrace();
        int idx = getIndexOfCaller(fqcn, st);
        if (idx < 0){
            this.stackTrace = st;
        }else{
            this.stackTrace = new StackTraceElement[st.length - idx];
            for (int i = 0; i < this.stackTrace.length; i++){
                this.stackTrace[i] = st[i + idx];
            }
        }
    }

    private static int getIndexOfCaller(String fqcn, StackTraceElement[] st)
    {
        for (int i = 0; i < st.length; i++){
            if (st[i].getClassName().equals(fqcn)){
                for (; i < st.length; i++){
                    if (!st[i].getClassName().equals(fqcn)){
                        return i;
                    }
                }
                return -1;
            }
        }
        return -1;
    }

    public String getLoggerName()
    {
        return loggerName;
    }

    public LogEvent setLoggerName(String loggerName)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace);
    }

    public Priority getPriority()
    {
        return priority;
    }

    public LogEvent setPriority(Priority priority)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace);
    }

    public String getMessage()
    {
        return message;
    }

    public LogEvent setMessage(String message)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace);
    }

    public Throwable getThrowable()
    {
        return throwable;
    }

    public LogEvent setThrowable(Throwable throwable)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace);
    }

    public long getTime()
    {
        return time;
    }

    public LogEvent setTime(long time)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace);
    }

    public Date getDate()
    {
        return new Date(time);
    }

    public LogEvent setDate(Date date)
    {
        return new LogEvent(loggerName, priority, message, throwable, date.getTime(), thread, stackTrace);
    }

    public Thread getThread()
    {
        return thread;
    }

    public LogEvent setThread(Thread thread)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace);
    }

    public StackTraceElement[] getStackTrace()
    {
        return stackTrace.clone();
    }

    public LogEvent setStackTrace(StackTraceElement[] stackTrace)
    {
        return new LogEvent(loggerName, priority, message, throwable, time, thread, stackTrace.clone());
    }
}
