package net.rumati.logging.muffero;

import java.util.Map;
import java.util.TreeMap;
import net.rumati.logging.muffero.util.ConservativeInheritableThreadLocal;

public class MDC
{
    private static final ConservativeInheritableThreadLocal<Map<String, String>> contextMaps = new ConservativeInheritableThreadLocal<Map<String, String>>() {

        @Override
        protected Map<String, String> create()
        {
            return new TreeMap<String, String>();
        }
    };

    private MDC(){}

    public static void put(String key, String value)
    {
        contextMaps.get(true).put(key, value);
    }

    public static String get(String key)
    {
        Map<String, String> map = contextMaps.get();
        if (map == null){
            return null;
        }else{
            return map.get(key);
        }
    }

    public static void remove(String key)
    {
        Map<String, String> map = contextMaps.get();
        if (map == null){
            return;
        }

        map.remove(key);
        if (map.isEmpty()){
            contextMaps.remove();
        }
    }

    public static void clear()
    {
        contextMaps.remove();
    }

    public static Map<String, String> getCopyOfContextMap()
    {
        Map<String, String> map = contextMaps.get();
        if (map == null){
            return new TreeMap<String, String>();
        }else{
            return new TreeMap<String, String>(map);
        }
    }

    public static void setContextMap(Map<String, String> map)
    {
        contextMaps.get(true).putAll(map);
    }
}
