package net.rumati.logging.muffero.appender.filter;

import net.rumati.logging.muffero.Appender;
import net.rumati.logging.muffero.LogEvent;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.Priority;

public class PriorityFilterAppender
        implements Appender
{
    private final Appender appender;
    private final Priority priorityThreshold;

    public PriorityFilterAppender(Appender appender, Priority priorityThreshold)
    {
        this.appender = appender;
        this.priorityThreshold = priorityThreshold;
    }

    public void append(LogEvent event)
    {
        if (event.getPriority() == priorityThreshold || event.getPriority().isMoreCriticalThan(priorityThreshold)){
            appender.append(event);
        }
    }

    public boolean isLevelEnabled(Priority p)
    {
        if (p == priorityThreshold || p.isMoreCriticalThan(priorityThreshold)){
            return true;
        }
        return false;
    }

    public void shutdown()
    {
        appender.shutdown();
    }
}
