package net.rumati.logging.muffero.layout;

import java.util.List;
import net.rumati.logging.muffero.Layout;
import net.rumati.logging.muffero.LogEvent;

/**
 * Uses a <em>formatting pattern</em> to encode data
 * from a {@link LogEvent} into a string, by replacing <em>formatting directives</em>
 * in the <em>formatting pattern</em> with data from the {@link LogEvent}. <em>Formatting
 * patterns</em> are made up of plain text, and <em>formatting directives</em>. The plain text
 * parts of the pattern are encoded into the output log message <em>as is</em>, without any modifications.
 * @author Jesse Long
 */
public class PatternLayout
        implements Layout
{
    private final List<PatternParser.Formatter> formatters;
    private final String pattern;

    public PatternLayout(String pattern)
    {
        this.pattern = pattern;
        this.formatters = new PatternParser(pattern).parse();
    }

    @Override
    public String toString()
    {
        return "PatternLayout: " + pattern;
    }

    public String format(LogEvent event)
    {
        StringBuilder sb = new StringBuilder(100);
        for (PatternParser.Formatter f : formatters){
            sb.append(f.getFormattedText(event));
        }
        return sb.toString();
    }
}
