package net.rumati.logging.muffero.util;

import java.util.Iterator;
import java.util.concurrent.locks.ReadWriteLock;

public class ReadWriteLockedIterator<E>
    implements Iterator<E>
{
    private final Iterator<E> it;
    protected final ReadWriteLock lock;

    public ReadWriteLockedIterator(Iterator<E> it, ReadWriteLock lock)
    {
        this.it = it;
        this.lock = lock;
    }

    public boolean hasNext()
    {
        lock.readLock().lock();
        try{
            return it.hasNext();
        }finally{
            lock.readLock().unlock();
        }
    }

    public E next()
    {
        lock.readLock().lock();
        try{
            return it.next();
        }finally{
            lock.readLock().unlock();
        }
    }

    public void remove()
    {
        lock.writeLock().lock();
        try{
            it.remove();
        }finally{
            lock.writeLock().unlock();
        }
    }
}
