package net.rumati.logging.muffero.util;

import java.util.Collection;
import java.util.List;
import java.util.ListIterator;
import java.util.concurrent.locks.ReadWriteLock;

public class ReadWriteLockedList<E>
    extends ReadWriteLockedCollection<E>
    implements List<E>
{
    private final List<E> list;

    public ReadWriteLockedList(List<E> list)
    {
        super(list);
        this.list = list;
    }

    public ReadWriteLockedList(List<E> list, ReadWriteLock lock)
    {
        super(list, lock);
        this.list = list;
    }

    public boolean addAll(int index,
                          Collection<? extends E> c)
    {
        lock.writeLock().lock();
        try {
            return list.addAll(index, c);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public E get(int index)
    {
        lock.readLock().lock();
        try {
            return list.get(index);
        }finally{
            lock.readLock().unlock();
        }
    }

    public E set(int index, E element)
    {
        lock.writeLock().lock();
        try {
            return list.set(index, element);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public void add(int index, E element)
    {
        lock.writeLock().lock();
        try {
            list.add(index, element);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public E remove(int index)
    {
        lock.writeLock().lock();
        try {
            return list.remove(index);
        }finally{
            lock.writeLock().unlock();
        }
    }

    public int indexOf(Object o)
    {
        lock.readLock().lock();
        try {
            return list.indexOf(o);
        }finally{
            lock.readLock().unlock();
        }
    }

    public int lastIndexOf(Object o)
    {
        lock.readLock().lock();
        try {
            return list.lastIndexOf(o);
        }finally{
            lock.readLock().unlock();
        }
    }

    public ListIterator<E> listIterator()
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedListIterator<E>(list.listIterator(), lock);
        }finally{
            lock.readLock().unlock();
        }
    }

    public ListIterator<E> listIterator(int index)
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedListIterator<E>(list.listIterator(index), lock);
        }finally{
            lock.readLock().unlock();
        }
    }

    public List<E> subList(int fromIndex, int toIndex)
    {
        lock.readLock().lock();
        try {
            return new ReadWriteLockedList<E>(list.subList(fromIndex, toIndex), lock);
        }finally{
            lock.readLock().unlock();
        }
    }
}
