package net.rumati.logging.muffero.util;

import java.util.ListIterator;
import java.util.concurrent.locks.ReadWriteLock;

public class ReadWriteLockedListIterator<E>
    extends ReadWriteLockedIterator<E>
    implements ListIterator<E>
{
    private final ListIterator<E> it;

    public ReadWriteLockedListIterator(ListIterator<E> it, ReadWriteLock lock)
    {
        super(it, lock);
        this.it = it;
    }

    public boolean hasPrevious()
    {
        lock.readLock().lock();
        try {
            return it.hasPrevious();
        }finally{
            lock.readLock().unlock();
        }
    }

    public E previous()
    {
        lock.readLock().lock();
        try {
            return it.previous();
        }finally{
            lock.readLock().unlock();
        }
    }

    public int nextIndex()
    {
        lock.readLock().lock();
        try {
            return it.nextIndex();
        }finally{
            lock.readLock().unlock();
        }
    }

    public int previousIndex()
    {
        lock.readLock().lock();
        try {
            return it.previousIndex();
        }finally{
            lock.readLock().unlock();
        }
    }

    public void set(E e)
    {
        lock.writeLock().lock();
        try {
            it.set(e);
        }finally{
            lock.readLock().unlock();
        }
    }

    public void add(E e)
    {
        lock.writeLock().lock();
        try {
            it.add(e);
        }finally{
            lock.writeLock().unlock();
        }
    }

}
